<?php
/**
 * Copyright (c) 2014-present, Facebook, Inc. All rights reserved.
 *
 * You are hereby granted a non-exclusive, worldwide, royalty-free license to
 * use, copy, modify, and distribute this software in source code or binary
 * form for use in connection with the web services and APIs provided by
 * Facebook.
 *
 * As with any software that integrates with the Facebook platform, your use
 * of this software is subject to the Facebook Developer Principles and
 * Policies [http://developers.facebook.com/policy/]. This copyright notice
 * shall be included in all copies or substantial portions of the software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 */

namespace FacebookAdsTest\Object;

use FacebookAds\Object\Campaign;
use FacebookAds\Object\Fields\CampaignFields;

class CampaignTest extends AbstractCrudObjectTestCase {

  public function testCrud() {
    $campaign = new Campaign(null, $this->getConfig()->accountId);
    $campaign->{CampaignFields::NAME} = $this->getConfig()->testRunId;
    
    $this->assertCanCreate($campaign);
    $this->assertCanRead($campaign);
    $this->assertCanUpdate(
      $campaign,
      array(
        CampaignFields::NAME => $this->getConfig()->testRunId.' updated',
      ));
    $this->assertCanFetchConnection($campaign, 'getAdSets');
    $this->assertCanFetchConnection($campaign, 'getAds');
    $this->assertCanFetchConnection($campaign, 'getInsights');
    $this->assertCanFetchConnection($campaign, 'getInsightsAsync');

    $this->assertCanBeLabeled($campaign);
    $this->assertCanArchive($campaign);

    $this->assertCanDelete($campaign);
  }
}
